# Copyright 2022 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import api, fields, models


class AccountMove(models.Model):
    _inherit = "account.move"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    purchasing_centre_id = fields.Many2one(
        comodel_name="res.partner",
        string="Centrale d'achat",
        compute="_compute_purchasing_centre_id",
        store=True,
    )
    membership_number = fields.Char(
        string="N° Adhérent",
        compute="_compute_membership_number",
        store=True,
    )
    payment_balance_date = fields.Date(
        string="Date de solde de paiement",
        compute="_compute_payment_balance_date",
        store=True,
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends(
        "partner_shipping_id",
        "partner_shipping_id.purchasing_centre_id",
        "partner_shipping_id.parent_id",
        "partner_shipping_id.parent_id.purchasing_centre_id",
    )
    def _compute_purchasing_centre_id(self):
        for move in self:
            if move.partner_shipping_id.purchasing_centre_id:
                move.purchasing_centre_id = (
                    move.partner_shipping_id.purchasing_centre_id
                )
            elif move.partner_shipping_id.parent_id.purchasing_centre_id:
                move.purchasing_centre_id = (
                    move.partner_shipping_id.parent_id.purchasing_centre_id
                )
            else:
                move.purchasing_centre_id = False

    @api.depends(
        "partner_shipping_id",
        "partner_shipping_id.membership_number",
        "partner_shipping_id.parent_id",
        "partner_shipping_id.parent_id.membership_number",
    )
    def _compute_membership_number(self):
        for move in self:
            if move.partner_shipping_id.membership_number:
                move.membership_number = move.partner_shipping_id.membership_number
            elif move.partner_shipping_id.parent_id.membership_number:
                move.membership_number = (
                    move.partner_shipping_id.parent_id.membership_number
                )
            else:
                move.membership_number = False

    @api.depends("payment_state")
    def _compute_payment_balance_date(self):
        """
        Calcule la dernière date de paiement pour les factures et avoirs clients à l'état
        "payé" afin de conserver la date de solde de la facture.
        """
        for invoice in self:
            invoice.payment_balance_date = False
            if (
                invoice.move_type in ["out_invoice", "out_refund"]
                and invoice.payment_state == "paid"
            ):
                payment_dates = invoice._get_reconciled_payments().mapped("date")
                statement_dates = invoice._get_reconciled_statement_lines().mapped(
                    "date"
                )
                reversed_dates = invoice._get_reconciled_invoices().mapped(
                    "invoice_date"
                )
                all_dates = payment_dates + statement_dates + reversed_dates
                if all_dates:
                    invoice.payment_balance_date = max(all_dates)

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
